# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS model importers (webservices API)
"""

from sqlalchemy.orm.exc import NoResultFound

from corepos.db.office_op import model as corepos
from corepos.api import CoreWebAPI

from rattail import importing


class ToCore(importing.ToSQLAlchemy):
    """
    Base class for all CORE "operational" model importers.

    Note that this class inherits from
    :class:`~rattail:rattail.importing.sqlalchemy.ToSQLAlchemy` even though our
    goal is to *not* write directly to the CORE DB.  However, (for now)
    importers will need to override methods to ensure API is used instead,
    where applicable.  But even once all have been refactored to write via API,
    we *still* may want to keep using ``ToSQLAlchemy`` for the sake of reading
    "cached local" data.  (May depend on how robust the API is.)
    """
    # TODO: should we standardize on the 'id' primary key? (can we even?)
    # key = 'id'

    def setup(self):
        self.establish_api()

    def establish_api(self):
        url = self.config.require('corepos.api', 'url')
        self.api = CoreWebAPI(url)

    # TODO: this looks an awful lot like it belongs in rattail proper
    def get_single_local_object(self, key):
        """
        Fetch a particular record from CORE, via SQLAlchemy.  This is used by
        the Rattail -> CORE datasync consumer.
        """
        query = self.session.query(self.model_class)
        for i, field in enumerate(self.key):
            query = query.filter(getattr(self.model_class, field) == key[i])

        for option in self.cache_query_options():
            query = query.options(option)

        try:
            return query.one()
        except NoResultFound:
            pass


class VendorImporter(ToCore):
    """
    Vendor model importer for CORE-POS
    """
    model_class = corepos.Vendor
    key = 'vendorID'
    supported_fields = [
        'vendorID',
        'vendorName',
        'vendorAbbreviation',
        'discountRate',
    ]
    # TODO: this importer is in a bit of an experimental state at the moment.
    # we only allow "update" b/c it will use the API instead of direct DB
    allow_create = False
    allow_delete = False

    def normalize_local_object(self, vendor):
        return {
            'vendorID': str(vendor.id),
            'vendorName': vendor.name,
            'vendorAbbreviation': vendor.abbreviation,
            'discountRate': vendor.discount_rate,
        }

    def update_object(self, vendor, data, local_data=None):
        """
        Push an update for the vendor, via the CORE API.
        """
        if self.dry_run:
            return data

        vendorID = data.pop('vendorID')
        self.api.set_vendor(vendorID, **data)
        return data
